const width = 1000;
const height = 1000;
const radius = Math.min(width, height) / 2 * 0.8;
const colorScale = d3.scaleOrdinal(d3.schemeCategory20b);

// total number of probes ie. leaf nodes
const numOfProbes = treeData.children.reduce((acc, curr) => {
    return acc + curr.children.length;
}, 0);

// set value for probe nodes
treeData.children.forEach(ph => {
    ph.children.forEach(ch => (ch.value = 1));
});

treeData.children.sort((a,b) => b.as - a.as)



document.querySelector("#msm-id").innerText = 28276188
document.querySelector("#msm-target").innerText = treeData.name
document.querySelector("#msm-probes").innerText = numOfProbes
document.querySelector("#msm-ph").innerText = treeData.children.length






const layout = d3.partition().size([2 * Math.PI, radius]);

const arcGenerator = d3
    .arc()
    .startAngle(d => d.x0)
    .endAngle(d => d.x1)
    .innerRadius(d => (d.depth === 1 ? d.y0 : d.y1 * 0.87))
    .outerRadius(d => (d.depth === 1 ? d.y1 * 1.3 : d.y1));

const root = d3.hierarchy(treeData)
  .sum(d => d.value)
const nodes = root.descendants();
layout(root);

const rotateText = d => {
    const angle = ((d.x0 + d.x1) / Math.PI) * 90;
    return angle < 180 ? angle - 90 : angle + 90;
};



const svgEl = document.querySelector('svg')
const phDetailsEl = document.querySelector('#ph-details')
const phProbeListEl = document.querySelector('#ph-probelist')


let phSelected = false

const mouseover = d => {
  if (phSelected) {
    return;
  }
  const sequence = getAncestors(d);
  d3.selectAll('.slice-group').style('opacity', 0.3);
  arcs.filter(node => sequence.indexOf(node) >= 0).style('opacity', 1);
  if (d.data.children) {
    const hoverPh = new CustomEvent('hover-ph', { detail: d.data });
    svgEl.dispatchEvent(hoverPh)
    document.querySelector("#ph-selected").innerText = d.data.hostname || d.data.name 
    document.querySelector("#ph-probes").innerText = d.data.children.length

    // remove existing rows
    while (phProbeListEl.firstChild) {
        phProbeListEl.removeChild(phProbeListEl.firstChild);
    }

    d.data.children.forEach(probe => {
      // add current rows
      const probeRow = document.createElement('tr')
      const probeIdCell = document.createElement('td')
      probeIdCell.innerText = probe.probeId
      const probeNameCell = document.createElement('td')
      probeNameCell.innerText = probe.name
      probeRow.appendChild(probeIdCell)
      probeRow.appendChild(probeNameCell)
      phProbeListEl.appendChild(probeRow)
    })
  }
  phDetailsEl.style.display = 'block';
  phProbeListEl.style.display = 'block';




};

const mouseleave = d => {
    if (phSelected) {
      return;
    }
    d3.selectAll('.slice-group').style('opacity', 1);
    phDetailsEl.style.display = 'none';
};


const mouseclick = d => {
  phSelected = !phSelected
}



const getAncestors = node => {
    let path = [];
    let current = node;
    while (current.parent) {
        path.unshift(current);
        current = current.parent;
    }
    return path;
};

// top level SVG container
const containerGroup = d3
    .select('svg')
    .attr('width', width)
    .attr('height', height)
    .append('g')
    .attr('id', 'container')
    .attr(
        'transform',
        'translate(' + width / 2 + ',' + height / 2 + ')scale(0.9)'
    )
    .on('mouseleave', mouseleave);

// slices
//
const arcsGroup = containerGroup.append('g').attr('id', 'arcs-group')

const arcs = arcsGroup 
    .selectAll('g')
    .data(nodes)
    .enter()
    .append('g')
    .filter(d => d.parent)
    .classed('slice-group', true);

arcs.append('path')
    .filter(d => d.parent)
    .attr('d', arcGenerator)
    .classed('arc', d => d.children)
    .style('fill', d => colorScale((d.children ? d : d.parent).data.as))
    .on('mouseover', mouseover)
    .on('click', mouseclick);

// inner ring - penultimate hops' text
arcs.filter(d => d.depth < 2 && d.value > 20)
    .append('text')
    .attr(
        'transform',
        d => `translate(${arcGenerator.centroid(d)})rotate(${rotateText(d)})`
    )
    .attr('dx', -width/10)
    .attr('dy', '.5em')
    .style('font-size', '12px')
    .style('font-weight', 'bold')
    .style('font-family', 'monospace')
    .text(d => d.data.hostname || d.data.name);

// after node outer ring - amount per each ph
arcs.filter(d => d.depth < 2 && d.value > 20)
    .append('text')
    .attr(
        'transform',
        d =>
            `translate(${arcGenerator.centroid(d)[0] *
                1.78},${arcGenerator.centroid(d)[1] * 1.78})`
    )
    .attr('dx', '-10')
    .attr('dy', '0')
    .style('font-size', '12px')
    .style('font-weight', 'bold')
    .style('font-family', 'monospace')
    .text(d => d.value);

arcs.filter(d => d.depth < 2 && d.value > 20)
    .append('text')
    .attr(
        'transform',
        d =>
            `translate(${arcGenerator.centroid(d)[0] *
                1.78},${arcGenerator.centroid(d)[1] * 1.78})`
    )
    .attr('dx', '-15')
    .attr('dy', '12')
    .style('font-size', '12px')
    .style('font-weight', 'bold')
    .style('font-family', 'monospace')
    .text(
        d => `${Number.parseFloat((d.value / numOfProbes) * 100).toFixed(2)}%`
    );



// asn ring setup ==================================================
//
const asns = treeData.children.map(c => {
  c.value = c.children.length
  return c
})
asns.sort((a,b) => b.value - a.value)
console.log(asns)

let asnRingObj = {}
for (const entry of asns) {
  console.log(entry)
  if (asnRingObj[entry.as]) {
    asnRingObj[entry.as] += entry.value
  } else {
    asnRingObj[entry.as] = entry.value
  }
}
console.log(asnRingObj)
asnRingObj.sort

const pie = d3.pie()
  .value(d => d.value)
  .sort(null);
const arc = d3.arc()
  .startAngle(d => -d.startAngle - 0.01)
  .endAngle(d => -d.endAngle - 0.01)
  .innerRadius(radius + 60)
  .outerRadius(radius + 80);

const asnRingData = pie(d3.entries(asnRingObj))
console.log(asnRingData)


const asRingGroup = containerGroup.append('g').attr('id', 'asring-group');

const slices = asRingGroup 
    .selectAll('g')
    .data(asnRingData)
    .enter()
    .append('g')
    .classed('asn-slice-group', true);

slices
  .append('path')
  .attr('d', arc)
  .attr('fill', function(d){ return(colorScale(d.data.key)) })
  .attr("stroke", "#fff")
  .style("opacity", 0.7)

slices.filter(d => d.data.value > 50)
    .append('text')
    .attr(
        'transform',
      d => {
        const c = arc.centroid(d)
        return `translate(${c[0]*1.06},${c[1]*1.06})`
      }
    )
    .attr('dx', '-12')
    .style('font-size', '14px')
    .style('font-weight', 'bold')
    .style('font-family', 'monospace')
    .text(d => `AS${d.data.key}`);


// end of asn ring setup ==================================================

